#!/bin/bash

# This function requires these variables to be set:
# $1 - Path of logfile to look at
# $2 - Path of file to store log offsets in
# This function sets these variables, which define the range of lines to look at:
# - log_start
# - log_end
function lookAtLog {
    logfile=$1
    dbfile=$2
    ### First time setup (if required)
    if [ ! -f "$dbfile" ]; then
        # Get initial log position
        log_end=$(wc -l $logfile | cut -d ' ' -f 1)
        # Write initial log position
        echo $log_end > $dbfile
    fi
    # Get old log position
    log_start=$(cat $dbfile)
    # Get new log position
    log_end=$(wc -l $logfile | cut -d ' ' -f 1)
    # Write new log position into file
    echo $log_end > $dbfile
    # Make sure we do not have a log rotation going on
    # This is the case when log_start > log_end.
    if [ $log_end -lt $log_start ]; then
        # In this case we start reading the new file from its beginning
        log_start=0
    fi
}

# This function requires these variables to be set:
# $logfile - The file to inspect
# $log_start - The line offset where to start inspecting
# $log_end - The line offset where to stop inspecting
function getLogLines {
    sed -n "$log_start","$log_end"p $logfile
}
