#!/bin/bash
#
# This script is supposed to be called periodically.
# It goes through the last few lines of a postfix mail.info log and counts events.
# The resulting numbers are then reported in the simple check_mk format.
#

# Debugging option
#set -x

# Go into appropriate directory
cd `dirname $0`

# Include log watch functions
. ./logwatch.functions.sh

# Use helper function to get $log_start and $log_end for current log
lookAtLog "/var/log/mail.info" "postfix-logstats.db"


### Initialize counters ###
log_lines_total=0
log_lines_smtp=0
log_lines_smtpd=0

mo_sent=0
mo_deferred=0
mo_bncd_total=0
mo_bncd_spam=0
mo_bncd_reputation=0
mo_deliverable=0
mo_undeliverable=0

mi_accept=0
mi_reject=0
mi_proxy_accept=0
mi_proxy_reject=0

mi_connects=0
mi_connects_tls=0

mi_dmarc_pass=0
mi_dmarc_fail=0
mi_dmarc_none=0

mi_dkim_signed=0
mi_dkim_error=0
mo_dkim_signed=0
mo_dkim_unsigned=0


### Do the statistics ###

# Collect number of log lines
log_lines_total=`expr $log_end - $log_start`
log_lines_smtp=$(getLogLines | grep '\/smtp\[' | wc -l)
log_lines_smtpd=$(getLogLines | grep '\/smtpd\[' | wc -l)

# Collect outbound email statistics - localhost is excluded.
mo_sent=$(              getLogLines | grep '\/smtp\[' | grep 'status=sent'          | grep -v 'relay=127\.0\.0\.1\|relay=localhost\|relay=::1' | wc -l)
mo_deferred=$(          getLogLines | grep '\/smtp\[' | grep 'status=deferred'      | grep -v 'relay=127\.0\.0\.1\|relay=localhost\|relay=::1' | wc -l)
mo_deliverable=$(       getLogLines | grep '\/smtp\[' | grep 'status=deliverable'   | grep -v 'relay=127\.0\.0\.1\|relay=localhost\|relay=::1' | wc -l)
mo_undeliverable=$(     getLogLines | grep '\/smtp\[' | grep 'status=undeliverable' | grep -v 'relay=127\.0\.0\.1\|relay=localhost\|relay=::1' | wc -l)
mo_bncd_total=$(        getLogLines | grep '\/smtp\[' | grep 'status=bounced'       | grep -v 'relay=127\.0\.0\.1\|relay=localhost\|relay=::1' | wc -l)
mo_bncd_spam=$(         getLogLines | grep '\/smtp\[' | grep 'status=bounced'       | grep -i 'spam' | grep -v 'relay=127\.0\.0\.1\|relay=localhost\|relay=::1' | wc -l)
mo_bncd_reputation=$(   getLogLines | grep '\/smtp\[' | grep -i 'reputation'        | grep -v 'relay=127\.0\.0\.1\|relay=localhost\|relay=::1' | wc -l)

# Collect inbound email statistics
mi_accept=$(getLogLines | grep '\/smtpd\[' | grep '\/smtpd\[' | grep ': [0-9A-F]*: client=' | wc -l)
mi_reject=$(getLogLines | grep '\/smtpd\[' | grep ': NOQUEUE: reject:' | wc -l)
mi_proxy_accept=$(getLogLines | grep '\/smtpd\[' | grep ': proxy-accept: END-OF-MESSAGE: ' | wc -l)
mi_proxy_reject=$(getLogLines | grep '\/smtpd\[' | grep ': proxy-reject: END-OF-MESSAGE: ' | wc -l)

# Collect connection data
mi_connects=$(getLogLines | grep '\/smtpd\[' | grep ': connect from' | wc -l)
mi_connects_tls=$(getLogLines | grep '\/smtpd\[' | grep 'Anonymous TLS connection established from' | wc -l)

# Collect DMARC data
mi_dmarc_pass=$(getLogLines | grep 'opendmarc\[' | grep 'pass$' | wc -l)
mi_dmarc_fail=$(getLogLines | grep 'opendmarc\[' | grep 'fail$' | wc -l)
mi_dmarc_none=$(getLogLines | grep 'opendmarc\[' | grep 'none$' | wc -l)

# Collect DKIM data
mi_dkim_signed=$(getLogLines | grep 'opendkim\[' | grep 'SSL$\|message has signatures from' | wc -l)
mi_dkim_error=$(getLogLines | grep 'opendkim\[' | grep -v 'SSL$\|message has signatures from\|DKIM-Signature field added\|no signing table match for' | wc -l)
mo_dkim_signed=$(getLogLines | grep 'opendkim\['| grep 'DKIM-Signature field added' | wc -l)
mo_dkim_unsigned=$(getLogLines | grep 'opendkim\['| grep 'no signing table match for' | wc -l)

### Report counter results ###
echo "P Postfix-logstats sent=$mo_sent|deferred=$mo_deferred|deliverable=$mo_deliverable|undeliverable=$mo_undeliverable|bounced=$mo_bncd_total|bounced-spam=$mo_bncd_spam|bounced-reputation=$mo_bncd_reputation|accepted=$mi_accept|proxy-accepted=$mi_proxy_accept|rejected=$mi_reject|proxy-rejected=$mi_proxy_reject|connections-in=$mi_connects|tls-connections-in=$mi_connects_tls|dmarc-pass=$mi_dmarc_pass|dmarc-fail=$mi_dmarc_fail|dmarc-none=$mi_dmarc_none|dkim-signed-in=$mi_dkim_signed|dkim-error-in=$mi_dkim_error|dkim-signed-out=$mo_dkim_signed|dkim-unsigned-out=$mo_dkim_unsigned Additional metrics related to postfix"

# That's it.
exit 0
